require "uri"

module OSDNClient
  class DefaultApi
    attr_accessor :api_client

    def initialize(api_client = ApiClient.default)
      @api_client = api_client
    end

    # 
    # Get access token by access code.
    # @param client_id Your application ID
    # @param client_secret Your application secret (key)
    # @param code Set auth code issued by https://sf-test-sugi.office.osdn.jp/account/oauth2ui/authorize
    # @param [Hash] opts the optional parameters
    # @option opts [String] :grant_type You need to set this as \&quot;authorization_code\&quot;
    # @return [Token]
    def token(client_id, client_secret, code, opts = {})
      data, status_code, headers = token_with_http_info(client_id, client_secret, code, opts)
      return data
    end

    # 
    # Get access token by access code.
    # @param client_id Your application ID
    # @param client_secret Your application secret (key)
    # @param code Set auth code issued by https://sf-test-sugi.office.osdn.jp/account/oauth2ui/authorize
    # @param [Hash] opts the optional parameters
    # @option opts [String] :grant_type You need to set this as \&quot;authorization_code\&quot;
    # @return [Array<(Token, Fixnum, Hash)>] Token data, response status code and response headers
    def token_with_http_info(client_id, client_secret, code, opts = {})
      if @api_client.config.debugging
        @api_client.config.logger.debug "Calling API: DefaultApi#token ..."
      end
      
      # verify the required parameter 'client_id' is set
      fail "Missing the required parameter 'client_id' when calling token" if client_id.nil?
      
      # verify the required parameter 'client_secret' is set
      fail "Missing the required parameter 'client_secret' when calling token" if client_secret.nil?
      
      # verify the required parameter 'code' is set
      fail "Missing the required parameter 'code' when calling token" if code.nil?
      
      # resource path
      path = "/token".sub('{format}','json')

      # query parameters
      query_params = {}

      # header parameters
      header_params = {}

      # HTTP header 'Accept' (if needed)
      _header_accept = []
      _header_accept_result = @api_client.select_header_accept(_header_accept) and header_params['Accept'] = _header_accept_result

      # HTTP header 'Content-Type'
      _header_content_type = ['application/x-www-form-urlencoded']
      header_params['Content-Type'] = @api_client.select_header_content_type(_header_content_type)

      # form parameters
      form_params = {}
      form_params["client_id"] = client_id
      form_params["client_secret"] = client_secret
      form_params["code"] = code
      form_params["grant_type"] = opts[:'grant_type'] if opts[:'grant_type']

      # http body (model)
      post_body = nil
      

      auth_names = []
      data, status_code, headers = @api_client.call_api(:POST, path,
        :header_params => header_params,
        :query_params => query_params,
        :form_params => form_params,
        :body => post_body,
        :auth_names => auth_names,
        :return_type => 'Token')
      if @api_client.config.debugging
        @api_client.config.logger.debug "API called: DefaultApi#token\nData: #{data.inspect}\nStatus code: #{status_code}\nHeaders: #{headers}"
      end
      return data, status_code, headers
    end
  end
end




