require 'date'

module OSDNClient
  class User
    attr_accessor :id

    attr_accessor :name

    attr_accessor :display_name

    attr_accessor :unix_status

    attr_accessor :unix_uid

    attr_accessor :authorized_keys

    attr_accessor :timezone

    attr_accessor :last_update

    attr_accessor :add_date

    attr_accessor :language

    attr_accessor :groups

    attr_accessor :chambers

    attr_accessor :skills

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        
        :'id' => :'id',
        
        :'name' => :'name',
        
        :'display_name' => :'display_name',
        
        :'unix_status' => :'unix_status',
        
        :'unix_uid' => :'unix_uid',
        
        :'authorized_keys' => :'authorized_keys',
        
        :'timezone' => :'timezone',
        
        :'last_update' => :'last_update',
        
        :'add_date' => :'add_date',
        
        :'language' => :'language',
        
        :'groups' => :'groups',
        
        :'chambers' => :'chambers',
        
        :'skills' => :'skills'
        
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        :'id' => :'Integer',
        :'name' => :'String',
        :'display_name' => :'String',
        :'unix_status' => :'BOOLEAN',
        :'unix_uid' => :'Integer',
        :'authorized_keys' => :'String',
        :'timezone' => :'String',
        :'last_update' => :'DateTime',
        :'add_date' => :'DateTime',
        :'language' => :'String',
        :'groups' => :'Array<SimpleGroup>',
        :'chambers' => :'Array<SimpleChamber>',
        :'skills' => :'Array<Skill>'
        
      }
    end

    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.inject({}){|memo,(k,v)| memo[k.to_sym] = v; memo}

      
      if attributes[:'id']
        self.id = attributes[:'id']
      end
      
      if attributes[:'name']
        self.name = attributes[:'name']
      end
      
      if attributes[:'display_name']
        self.display_name = attributes[:'display_name']
      end
      
      if attributes[:'unix_status']
        self.unix_status = attributes[:'unix_status']
      end
      
      if attributes[:'unix_uid']
        self.unix_uid = attributes[:'unix_uid']
      end
      
      if attributes[:'authorized_keys']
        self.authorized_keys = attributes[:'authorized_keys']
      end
      
      if attributes[:'timezone']
        self.timezone = attributes[:'timezone']
      end
      
      if attributes[:'last_update']
        self.last_update = attributes[:'last_update']
      end
      
      if attributes[:'add_date']
        self.add_date = attributes[:'add_date']
      end
      
      if attributes[:'language']
        self.language = attributes[:'language']
      end
      
      if attributes[:'groups']
        if (value = attributes[:'groups']).is_a?(Array)
          self.groups = value
        end
      end
      
      if attributes[:'chambers']
        if (value = attributes[:'chambers']).is_a?(Array)
          self.chambers = value
        end
      end
      
      if attributes[:'skills']
        if (value = attributes[:'skills']).is_a?(Array)
          self.skills = value
        end
      end
      
    end

    # Check equality by comparing each attribute.
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          id == o.id &&
          name == o.name &&
          display_name == o.display_name &&
          unix_status == o.unix_status &&
          unix_uid == o.unix_uid &&
          authorized_keys == o.authorized_keys &&
          timezone == o.timezone &&
          last_update == o.last_update &&
          add_date == o.add_date &&
          language == o.language &&
          groups == o.groups &&
          chambers == o.chambers &&
          skills == o.skills
    end

    # @see the `==` method
    def eql?(o)
      self == o
    end

    # Calculate hash code according to all attributes.
    def hash
      [id, name, display_name, unix_status, unix_uid, authorized_keys, timezone, last_update, add_date, language, groups, chambers, skills].hash
    end

    # build the object from hash
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map{ |v| _deserialize($1, v) } )
          else
            #TODO show warning in debug mode
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        else
          # data not found in attributes(hash), not an issue as the data can be optional
        end
      end

      self
    end

    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value.to_s =~ /^(true|t|yes|y|1)$/i
          true
        else
          false
        end
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k)] = _deserialize(v_type, v)
          end
        end
      else # model
        _model = OSDNClient.const_get(type).new
        _model.build_from_hash(value)
      end
    end

    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_body (backward compatibility))
    def to_body
      to_hash
    end

    # return the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      hash
    end

    # Method to output non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map{ |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end

  end
end
